program gen;

{$APPTYPE CONSOLE}
{$R+}

uses
	SysUtils, Types;

function SameL(const A, B: string; Len: Integer): Boolean;
begin
	Result := (Length(A) >= Len) and (Length(B) >= Len) and CompareMem(Pointer(A), Pointer(B), Len * SizeOf(AnsiChar));
end;

function WideSameL(const A, B: WideString; Len: Integer): Boolean;
begin
	Result := (Length(A) >= Len) and (Length(B) >= Len) and CompareMem(Pointer(A), Pointer(B), Len * SizeOf(WideChar));
end;

function Escape(C: Char): string;
begin
	if C in [' ', '^'] then
		Result := ''
	else if not (C in ['A'..'Z']) and not (C in ['a'..'z']) then
		Result := '\' + C
	else
		Result := C
end;

procedure MakeKana;
	
	type
		TC = (cNone, cK, cS, cT, cN, cH, cM, cY, cR, cW, cG, cZ, cD, cB, cP, cV, cQ, cJ, cC, cF);
		TS = (sNone, sIX, sUX, sEX);
		TM = (mA, mI, mU, mE, mO, mYA, mYU, mYO);
		TL = (lNone, lLittle);
		TK = (kNone, kTail, kAll);
		TT = (tNone, tTSU);
	
	const
		CKey : array[TC] of string = (' ', 'k', 's', 't', 'n', 'h', 'm', 'y', 'r', 'w', 'g', 'z', 'd', 'b', 'p', 'v', 'q', 'j', 'c', 'f');
		SKey : array[TS] of string = (' ', 'y', 'w', 'h');
		MKey : array[TM] of string = ('a', 'i', 'u', 'e', 'o', '', '', '');
		LKey : array[TL] of string = (' ', 'l');
	
	function Shift(C: Char): Char;
	begin
		case C of
			';': Result := '+';
			':': Result := '*';
			',': Result := '<';
			'.': Result := '>';
			'/': Result := '?';
			'''': Result := '"';
			else Result := UpCase(C);
		end
	end;
	
	function ToKatakana(S : WideString) : WideString;
	const
		H: WideString = 'ĂƂȂɂʂ˂̂͂Ђӂւق܂݂ނ߂񂪂Âłǂ΂тԂׂڂς҂Ղ؂ۂ';
		K: WideString = 'ACEGIJLNPRTVXZ\^`cegijklmnqtwz}~KMOQSUWY[]_adfhorux{psvy|@BDFHb';
	var
		I, P: Integer;
	begin
		SetLength(Result, Length(S));
		for I := 1 to Length(S) do
		begin
			P := Pos(S[I], H);
			if P > 0 then
				Result[I] := K[P]
			else
				Result[I] := S[I];
		end;
	end;
	
	function Sequence(C: TC; S: TS; M: TM; L: TL; T: TT; K: TK): TStringDynArray;
	var
		CI, SI, MI, LI: Integer;
		R: string;
	begin
		Result := nil;
		for CI := 1 to Length(CKey[C]) do
			for SI := 1 to Length(SKey[S]) do
				for MI := 1 to Length(MKey[M]) do
					for LI := 1 to Length(LKey[L]) do
					begin
						if (LKey[L, LI] = '^') or (K = kAll) then
							R := Escape(Shift(LKey[L, LI])) + Escape(Shift(CKey[C, CI])) + Escape(Shift(SKey[S, SI])) + Escape(Shift(MKey[M, MI]))
						else if K = kTail then
						begin
							R := Escape(LKey[L, LI]) + Escape(CKey[C, CI]) + Escape(SKey[S, SI]) + Escape(Shift(MKey[M, MI]));
							if R = Escape(Shift(LKey[L, LI])) + Escape(Shift(CKey[C, CI])) + Escape(Shift(SKey[S, SI])) + Escape(Shift(MKey[M, MI])) then
								Continue;
						end
						else
							R := Escape(LKey[L, LI]) + Escape(CKey[C, CI]) + Escape(SKey[S, SI]) + Escape(MKey[M, MI]);
						
						if T <> tNone then
							R := R[1] + R;
						
						SetLength(Result, Length(Result) + 1);
						Result[High(Result)] := R;
					end;
	end;
	
	function Compose(C: TC; S: TS; M: TM; L: TL; T: TT; K: TK): WideString;
		function W(S : WideString): PWideChar;
		begin
			Result := PWideChar(S);
		end;
	begin
		Result := '';
		if (C = cNone) and (S <> sNone) then Exit;
		if (C = cY) and (S = sIX) then Exit;
		if (C = cW) and (S = sUX) then Exit;
		if (C = cH) and (S = sEX) then Exit;
		case L of
			lNone:
				case S of
					sNone:
						case C of
							cNone: SetString(Result, W('@@@@@@@@') + (Ord(M) * 2), 2);
							cK   : SetString(Result, W('@@@@@Ⴋカ') + (Ord(M) * 2), 2);
							cS   : SetString(Result, W('@@@@@Ⴕサ') + (Ord(M) * 2), 2);
							cT   : SetString(Result, W('@@@ā@Ɓ@Ⴟタ') + (Ord(M) * 2), 2);
							cN   : SetString(Result, W('ȁ@Ɂ@ʁ@ˁ@́@ɂɂɂ') + (Ord(M) * 2), 2);
							cH   : SetString(Result, W('́@Ё@Ӂ@ց@ف@ЂЂЂ') + (Ord(M) * 2), 2);
							cM   : SetString(Result, W('܁@݁@ށ@߁@@݂݂݂') + (Ord(M) * 2), 2);
							cY   : SetString(Result, W('@@@@@@@@') + (Ord(M) * 2), 2);
							cR   : SetString(Result, W('@@@@@') + (Ord(M) * 2), 2);
							cW   : SetString(Result, W('@@@@@@@@') + (Ord(M) * 2), 2);
							cG   : SetString(Result, W('@@@@@Ⴌガ') + (Ord(M) * 2), 2);
							cZ   : SetString(Result, W('@@@@@Ⴖザ') + (Ord(M) * 2), 2);
							cD   : SetString(Result, W('@@Á@Ł@ǁ@') + (Ord(M) * 2), 2);
							cB   : SetString(Result, W('΁@с@ԁ@ׁ@ځ@ттт') + (Ord(M) * 2), 2);
							cP   : SetString(Result, W('ρ@ҁ@Ձ@؁@ہ@҂҂҂') + (Ord(M) * 2), 2);
							cV   : SetString(Result, W('@ეピ') + (Ord(M) * 2), 2);
							cQ   : SetString(Result, W('@Ⴍキ') + (Ord(M) * 2), 2);
							cJ   : SetString(Result, W('Ⴖ@ザ傶Ⴖザ') + (Ord(M) * 2), 2);
							cC   : SetString(Result, W('@@@@@Ⴟタ') + (Ord(M) * 2), 2);
							cF   : SetString(Result, W('ӂӂӁ@ӂӂӂӂӂ') + (Ord(M) * 2), 2);
						end;
					sIX: {Y}
						case C of
							cK   : SetString(Result, W('Ⴋカ傫Ⴋカ') + (Ord(M) * 2), 2);
							cS   : SetString(Result, W('Ⴕサ債Ⴕサ') + (Ord(M) * 2), 2);
							cT   : SetString(Result, W('Ⴟタ傿Ⴟタ') + (Ord(M) * 2), 2);
							cN   : SetString(Result, W('ɂɂɂɂɂɂɂɂ') + (Ord(M) * 2), 2);
							cH   : SetString(Result, W('ЂЂЂЂЂЂЂЂ') + (Ord(M) * 2), 2);
							cM   : SetString(Result, W('݂݂݂݂݂݂݂݂') + (Ord(M) * 2), 2);
							cR   : SetString(Result, W('股肥') + (Ord(M) * 2), 2);
							cG   : SetString(Result, W('Ⴌガ催Ⴌガ') + (Ord(M) * 2), 2);
							cZ   : SetString(Result, W('Ⴖザ傶Ⴖザ') + (Ord(M) * 2), 2);
							cD   : SetString(Result, W('') + (Ord(M) * 2), 2);
							cB   : SetString(Result, W('тттттттт') + (Ord(M) * 2), 2);
							cP   : SetString(Result, W('҂҂҂҂҂҂҂҂') + (Ord(M) * 2), 2);
							cV   : SetString(Result, W('тттეピ') + (Ord(M) * 2), 2);
							cQ   : SetString(Result, W('Ⴍキ傭Ⴍキ') + (Ord(M) * 2), 2);
							cJ   : SetString(Result, W('Ⴖザ傶Ⴖザ') + (Ord(M) * 2), 2);
							cC   : SetString(Result, W('Ⴟタ傿Ⴟタ') + (Ord(M) * 2), 2);
							cF   : SetString(Result, W('ӂӂӂӂӂӂӂӂ') + (Ord(M) * 2), 2);
						end;
					sUX: {W}
						case C of
							cK   : SetString(Result, W('Ⴍキ') + (Ord(M) * 2), 2);
							cS   : SetString(Result, W('Ⴗシ') + (Ord(M) * 2), 2);
							cT   : SetString(Result, W('ƂĂƂƂƂ@@@@@@') + (Ord(M) * 2), 2);
							cH   : SetString(Result, W('ӂӂӂӂӂӂӂӂ') + (Ord(M) * 2), 2);
							cG   : SetString(Result, W('Ⴎギ') + (Ord(M) * 2), 2);
							cZ   : SetString(Result, W('Ⴘジ') + (Ord(M) * 2), 2);
							cD   : SetString(Result, W('ǂłǂǂǂ@@@@@@') + (Ord(M) * 2), 2);
							cB   : SetString(Result, W('ԂԂԂԂԂԂԂԂ') + (Ord(M) * 2), 2);
						end;
					sEX: {H}
						case C of
							cK   : if Pos('c', CKey[cK]) > 0 then
								SetString(Result, W('Ⴟタ傿Ⴟタ') + (Ord(M) * 2), 2);
							cS   : SetString(Result, W('Ⴕ@サ@@@@@@') + (Ord(M) * 2), 2);
							cT   : SetString(Result, W('ĂĂĂĂĂĂĂĂ') + (Ord(M) * 2), 2);
							cW   : SetString(Result, W('@@@@@@@') + (Ord(M) * 2), 2);
							cZ   : SetString(Result, W('Ⴖ@ザ@@@@@@') + (Ord(M) * 2), 2);
							cD   : SetString(Result, W('łłłłłłłł') + (Ord(M) * 2), 2);
						end;
				end;
			lLittle:
				case S of
					sNone:
						case C of
							cNone: Result := W('')[Ord(M)];
							cK   : Result := W('@@@@@@')[Ord(M)];
							cY   : Result := W('@@@@@')[Ord(M)];
							cW   : Result := W('@@@@@@@')[Ord(M)];
						end;
				end;
		end;
		while (Result <> '') and (Result[Length(Result)] = '@') do
			Delete(Result, Length(Result), 1);
		if (Result <> '') and (T <> tNone) then
		begin
			if Pos(Result[1], 'Ȃɂʂ˂̂傟샕') = 0 then
				Result := '' + Result
			else
				Result := ''
		end;
		if K <> kNone then
			Result := ToKatakana(Result)
	end;
	
	var
		C: TC;
		S: TS;
		M: TM;
		L: TL;
		K: TK;
		T: TT;
		Kana: string;
		Seqs: TStringDynArray;
		I: Integer;
	begin
		for C := Low(TC) to High(TC) do
		begin
			for S := Low(TS) to High(TS) do
			begin
				for M := Low(TM) to High(TM) do
				begin
					for L := Low(TL) to High(TL) do
					begin
						for T := Low(TT) to High(TT) do
						begin
							for K := Low(TK) to High(TK) do
							begin
								Kana := Compose(C, S, M, L, T, K);
								if Kana <> '' then
								begin
									Seqs := Sequence(C, S, M, L, T, K);
									if Length(Seqs) > 0 then
									begin
										for I := Low(Seqs) to High(Seqs) do
										begin
											if K = kNone then
												WriteLn(Seqs[I], ' = ', Kana)
											else
												WriteLn(Seqs[I], ' /F = ', Kana);
										end
									end
								end
							end
						end;
					end
				end
			end
		end;
		WriteLn('n\'' = ');
		WriteLn('n\; = ');
		WriteLn('n\+ /F = ');
		WriteLn('N\+ /F = ');
		WriteLn('n\: /F = ');
		WriteLn('N\: /F = ');
		WriteLn('t\'' = ');
		WriteLn('t\; = ');
		WriteLn('t\+ /F = b');
		WriteLn('T\+ /F = b');
		WriteLn('t\: /F = b');
		WriteLn('T\: /F = b');
		WriteLn('m\'' = X');
		WriteLn('m\; = X');
		WriteLn('m\+ /F = X');
		WriteLn('M\+ /F = X');
		WriteLn('m\: /F = X');
		WriteLn('M\: /F = X');
end;

procedure MakeNIK50;
type
	TPri = record
		Part : WideString;
		Key : Char;
	end;
	TAlt = record
		Part : WideString;
		Seq : WideString;
	end;
	TCode = record
		Kanji : WideString;
		Seq : string;
	end;
const
	Primitives : array[1..50] of TPri = (
		(Part: ''; Key: 'Q'),
		(Part: ''; Key: 'q'),
		(Part: 'V'; Key: 'W'),
		(Part: 'q'; Key: 'w'),
		(Part: 'c'; Key: 'R'),
		(Part: '\'; Key: 'r'),
		(Part: ''; Key: 'T'),
		(Part: ''; Key: 't'),
		(Part: ''; Key: 'Y'),
		(Part: 'O'; Key: 'y'),
		(Part: ''; Key: 'P'),
		(Part: 'N'; Key: 'p'),
		(Part: ''; Key: 'S'),
		(Part: 'm'; Key: 's'),
		(Part: ''; Key: 'D'),
		(Part: ''; Key: 'd'),
		(Part: ''; Key: 'F'),
		(Part: 'E'; Key: 'f'),
		(Part: '~'; Key: 'G'),
		(Part: 'k'; Key: 'g'),
		(Part: ''; Key: 'H'),
		(Part: 'b'; Key: 'h'),
		(Part: ''; Key: 'J'),
		(Part: ''; Key: 'j'),
		(Part: ''; Key: 'K'),
		(Part: '\'; Key: 'k'),
		(Part: ''; Key: 'L'),
		(Part: 'R'; Key: 'l'),
		(Part: ''; Key: '+'),
		(Part: ''; Key: ';'),
		(Part: ''; Key: 'Z'),
		(Part: 'y'; Key: 'z'),
		(Part: ''; Key: 'X'),
		(Part: ''; Key: 'x'),
		(Part: ''; Key: 'C'),
		(Part: 'R'; Key: 'c'),
		(Part: ''; Key: 'V'),
		(Part: 'k'; Key: 'v'),
		(Part: 'a'; Key: 'B'),
		(Part: 's'; Key: 'b'),
		(Part: 'l'; Key: 'N'),
		(Part: 'C'; Key: 'n'),
		(Part: 'l'; Key: 'M'),
		(Part: 'n'; Key: 'm'),
		(Part: ''; Key: '<'),
		(Part: ''; Key: ','),
		(Part: ''; Key: '>'),
		(Part: 'g'; Key: '.'),
		(Part: 'T'; Key: '?'),
		(Part: ''; Key: '/'));
	
	Alts : array[1..27 - 5 + 52] of TAlt = (
		(Part: ''; Seq: 'kb'),
		(Part: ''; Seq: 'Rm'),
		//(Part: ''; Seq: 'c'),
		(Part: 'R'; Seq: 'b'),
		(Part: ''; Seq: '\'),
		(Part: ''; Seq: ''),
		(Part: 'Z'; Seq: 'C'),
		(Part: ''; Seq: '\'),
		(Part: '~'; Seq: 'g'),
		(Part: '|'; Seq: 'b'),
		(Part: ''; Seq: 'b'),
		(Part: ''; Seq: 'l'),
		(Part: 'J'; Seq: 'ʃm'),
		//(Part: 'm'; Seq: 'k'),
		(Part: ''; Seq: 'm'),
		(Part: '^'; Seq: 'iE'),
		(Part: 'c'; Seq: 'k'),
		(Part: 't'; Seq: ''),
		//(Part: 'a'; Seq: 'n'),
		(Part: 'i'; Seq: '\'),
		//(Part: ''; Seq: 'qq'),
		(Part: ''; Seq: 'k'),
		(Part: 'X'; Seq: 'n'),
		(Part: ''; Seq: 'cq'),
		(Part: 'c'; Seq: 'cy'),
		//(Part: 'O'; Seq: 'l'),
		(Part: ''; Seq: 'ʁb'),
		//ȍ~p
		(Part: ''; Seq: ''),
		(Part: '}'; Seq: ''),
		(Part: 'M'; Seq: 'R'),
		(Part: ''; Seq: 'R'),
		(Part: 'e'; Seq: 'mC'),
		(Part: '_'; Seq: 'kR'),
		(Part: 'q'; Seq: ''),
		(Part: 'm'; Seq: '\'),
		(Part: 'c'; Seq: 'mm'),
		(Part: ''; Seq: 'bb'),
		(Part: ''; Seq: ''),
		(Part: 'O'; Seq: ''),
		(Part: 'T'; Seq: 'RE'),
		(Part: ''; Seq: 'NR'),
		(Part: ''; Seq: 'NR'),
		(Part: ''; Seq: '\E'),
		(Part: 'H'; Seq: 's'),
		(Part: ''; Seq: 'm'),
		(Part: ''; Seq: 'cb'),
		(Part: ''; Seq: 'i'),
		(Part: ''; Seq: 'E'),
		(Part: 'n'; Seq: 'E'),
		(Part: '['; Seq: 'NE'),
		(Part: '|'; Seq: 'RT'),
		(Part: ''; Seq: 'l'),
		(Part: ''; Seq: ''),
		(Part: 'S'; Seq: 'EEE'),
		(Part: 'o'; Seq: 'nEE'),
		(Part: ''; Seq: '\'),
		(Part: ''; Seq: 's'),
		(Part: ''; Seq: 'l'),
		(Part: ''; Seq: 'O'),
		(Part: ''; Seq: 'ib'),
		(Part: ''; Seq: 'mb'),
		(Part: 'u'; Seq: 'Cs'),
		(Part: ''; Seq: 'E'),
		(Part: ''; Seq: 'm'),
		(Part: 'M'; Seq: 'c\'),
		(Part: 'l'; Seq: 'cn'),
		(Part: ''; Seq: 'cn'),
		(Part: ''; Seq: 'lE'),
		(Part: 'D'; Seq: 'cg'),
		(Part: ''; Seq: 'cn'),
		(Part: ''; Seq: '\'),
		(Part: 'p'; Seq: 'Nm'),
		(Part: 'g'; Seq: 'E'),
		(Part: ''; Seq: '\'),
		(Part: 'J'; Seq: 'cT'),
		(Part: ''; Seq: '\bn'),
		(Part: 'H'; Seq: 'lE'),
		(Part: 'n'; Seq: 'b'),
		(Part: ''; Seq: 'Nc'));
	
var
	Data: array of TCode;
	
	function KeyToPrim(C: Char): WideString;
	var
		I: Integer;
	begin
		for I := High(Primitives) downto Low(Primitives) do
			if Primitives[I].Key = C then
			begin
				Result := Primitives[I].Part;
				Exit;
			end;
		Assert(False);
	end;
	
	procedure PrimToKey(const Kanji: WideString; const PartSeq: WideString; const KeySeq: string);
	var
		I: Integer;
	begin
		if PartSeq = '' then
		begin
			for I := High(Data) downto Low(Data) do
			begin
				if Data[I].Seq = KeySeq then
				begin
					WriteLn(ErrOutput, 'Sequence of "' + Kanji + '" was duplicated with "' + Data[I].Kanji + '".');
					Exit;
				end;
			end;
			SetLength(Data, Length(Data) + 1);
			Data[High(Data)].Kanji := Kanji;
			Data[High(Data)].Seq := KeySeq;
		end
		else
		begin
			for I := High(Primitives) downto Low(Primitives) do
			begin
				if Primitives[I].Part = PartSeq[1] then
				begin
					PrimToKey(Kanji, Copy(PartSeq, 2), KeySeq + Primitives[I].Key);
					Exit;
				end;
			end;
			if (PartSeq[1] >= '0') and (PartSeq[1] <= '9') then
			begin
				PrimToKey(Kanji, Copy(PartSeq, 2), KeySeq + PartSeq[1]);
				Exit;
			end;
			for I := High(Alts) downto Low(Alts) do
			begin
				if Alts[I].Part = PartSeq[1] then
				begin
					PrimToKey(Kanji, Alts[I].Seq + Copy(PartSeq, 2), KeySeq);
					Exit;
				end;
			end;
			WriteLn(ErrOutput, '"' + Kanji + '" could not be converted.')
		end
	end;
	
var
	Dic, FreqR, LongR: Text;
	I, J, K, Len: Integer;
	C: WideChar;
	Kanji, S: WideString;
	Freq: array[1..50] of Integer;
	Double: Boolean;
	OrgS, ShortS: string;
	NextStroke: Char;
	Used: array[Char, Char] of Boolean;
begin
	Assign(Dic, 'nik50dic.txt');
	Reset(Dic);
	while not Eof(Dic) do
	begin
		Kanji := '';
		repeat
			Read(Dic, C);
			if C = ':' then Break;
			Kanji := Kanji + C;
		until False;
		ReadLn(Dic, S);
		PrimToKey(Kanji, S, '');
	end;
	Close(Dic);
	//ȗ̍쐬
	for Len := 63 downto 3 do
	begin
		for I := High(Data) downto Low(Data) do
		begin
			if Len = Length(Data[I].Seq) then
			begin
				OrgS := Data[I].Seq;
				Double := False;
				for J := High(Data) downto Low(Data) do
				begin
					Double := (I <> J) and (Length(Data[J].Seq) in [Len - 1 .. Len]) and SameL(Data[J].Seq, OrgS, Len - 1);
					if Double then Break;
				end;
				if not Double then
				begin
					ShortS := Copy(OrgS, 1, Len - 1);
					for J := High(Data) downto Low(Data) do
					begin
						if SameL(Data[J].Seq, OrgS, Len) then
						begin
							Data[J].Seq := ShortS + Copy(Data[J].Seq, Len + 1);
						end;
					end;
				end;
			end;
		end;
	end;
	//ȗpXȍ~̂SēL[ōXɂŊm肷鎚Ȃ炷΂
	for I := High(Data) downto Low(Data) do
	begin
		if Data[I].Kanji = '@' then
		begin
			NextStroke := #0;
			for J := Low(Data) to High(Data) do
			begin
				if (J <> I) and SameL(Data[J].Seq, Data[I].Seq, Length(Data[I].Seq)) then
				begin
					if NextStroke = #0 then
						NextStroke := Data[J].Seq[Length(Data[I].Seq) + 1]
					else if Data[J].Seq[Length(Data[I].Seq) + 1] <> NextStroke then
					begin
						NextStroke := #1;
						Break;
					end;
				end;
			end;
			if NextStroke > #32 then
			begin
				for J := Low(Data) to High(Data) do
				begin
					if (J <> I) and SameL(Data[J].Seq, Data[I].Seq, Length(Data[I].Seq)) then
					begin
						Delete(Data[J].Seq, Length(Data[I].Seq) + 1, 1);
					end;
				end;
			end;
		end;
	end;
	//ȗpXKChɕϊ
	for I := High(Data) downto Low(Data) do
	begin
		if Data[I].Kanji = '@' then
		begin
			Data[I].Kanji := '';
			for J := Low(Data) to High(Data) do
			begin
				if (J <> I) and SameL(Data[J].Seq, Data[I].Seq, Length(Data[I].Seq)) then
				begin
					K := Pos('(' + KeyToPrim(Data[J].Seq[Length(Data[I].Seq) + 1]) + ')', Data[I].Kanji);
					if K > 0 then
						Insert(Data[J].Kanji, Data[I].Kanji, K)
					else
						Data[I].Kanji := Data[I].Kanji + Data[J].Kanji + '(' + KeyToPrim(Data[J].Seq[Length(Data[I].Seq) + 1]) + ')';
				end;
			end;
		end;
	end;
	//pxf[^
	for K := 1 to 50 do Freq[K] := 0;
	for I := High(Data) downto Low(Data) do
	begin
		for J := 1 to Length(Data[I].Seq) do
		begin
			for K := 1 to 50 do 
				if Data[I].Seq[J] = Primitives[K].Key then
				begin
					Inc(Freq[K]);
					Break;
				end;
		end;
	end;
	//pxo
	Assign(FreqR, 'log-frequency.txt');
	Rewrite(FreqR);
	for K := Low(Primitives) to High(Primitives) do
	begin
		WriteLn(FreqR, Primitives[K].Part, ':', Freq[K]);
	end;
	Close(FreqR);
	//zo
	Assign(LongR, 'log-long.txt');
	Rewrite(LongR);
	for Len := 63 downto 6 do
		for I := High(Data) downto Low(Data) do
		begin
			if Length(Data[I].Seq) = Len then
				WriteLn(LongR, Data[I].Kanji, '(', Length(Data[I].Seq), ')');
		end;
	for Len := 5 downto 1 do
	begin
		K := 0;
		for I := High(Data) downto Low(Data) do
		begin
			if Length(Data[I].Seq) = Len then Inc(K);
		end;
		WriteLn(LongR, Len, '(', K, ')');
	end;
	FillChar(Used, SizeOf(Used), #0);
	for I := High(Data) downto Low(Data) do
		if Length(Data[I].Seq) >= 2 then
		begin
			Used[Data[I].Seq[1], Data[I].Seq[2]] := True;
		end;
	for I := Low(Primitives) to High(Primitives) do
	begin
		Write(LongR, KeyToPrim(Primitives[I].Key));
		for J := Low(Primitives) to High(Primitives) do
			if not Used[Primitives[I].Key, Primitives[J].Key] then
			begin
				Write(LongR, KeyToPrim(Primitives[J].Key));
			end;
		WriteLn(LongR);
	end;
	Close(LongR);
	//pL[{[hΉ
	for I := High(Data) downto Low(Data) do
	begin
		S := Data[I].Seq;
		S := StringReplace(S, '+', ':', [rfReplaceAll]);
		if S <> Data[I].Seq then
		begin
			SetLength(Data, Length(Data) + 1);
			Data[High(Data)].Kanji := Data[I].Kanji;
			Data[High(Data)].Seq := S;
		end;
	end;
	//o
	for I := Low(Data) to High(Data) do
	begin
		for J := 1 to Length(Data[I].Seq) do
		begin
			Write(Escape(Data[I].Seq[J]));
		end;
		Write(' /F = ');
		Write(Data[I].Kanji);
		WriteLn;
	end;
end;

begin
	Assign(Output, 'nik50.grx');
	
	Rewrite(Output);
	
	WriteLn('[Description]');
	WriteLn('NIK-50');
	WriteLn('NIK-Code in 30 Keys');
	WriteLn('');
	WriteLn('[Attribute]');
	WriteLn('');
	WriteLn('[Conversion]');
	
	WriteLn(', /HF = A C');
	WriteLn('. /HF = B D');
	WriteLn('- /F = [');
	WriteLn('-- /F = |');
	WriteLn('--- /F = ');
	WriteLn('/ /F = E');
	WriteLn('/// /F = c');
	WriteLn('~ /KF = `');
	WriteLn('[ /OF = u');
	WriteLn('] /CF = v');
	WriteLn('[[ /OF = w');
	WriteLn(']] /CF = x');
	WriteLn('[[[ /OF = y');
	WriteLn(']]] /CF = z');
	WriteLn('[\" /OF = g');
	WriteLn(']\" /CF = h');
	WriteLn('[< /FO = s');
	WriteLn('[> /FC = t');
	WriteLn('>= /F = ');
	WriteLn('<= /F = ');
	WriteLn('+- /F = }');
	WriteLn(':= /F = ');
	WriteLn('/= /F = ');
	WriteLn(':- /F = ');
	WriteLn('/\\ /F = ~');
	WriteLn('^== /F = ');
	WriteLn('v== /F = ');
	WriteLn('<== /F = ');
	WriteLn('>== /F = ');
	WriteLn('==> /F = ');
	WriteLn('==>> /F = ');
	//WriteLn('\: /F = [');
	WriteLn('\\: /F = F');
	WriteLn('\\\; /F = G');
	WriteLn('\\\* /F = ');
	WriteLn('\\\+ /F = {');
	WriteLn('\\**   /F = ');
	WriteLn('\\***  /F = ');
	WriteLn('\\**** /F = ');
	WriteLn('\\<>   /F = ');
	WriteLn('\\<>>  /F = ');
	WriteLn('\\[]   /F = ');
	WriteLn('\\[]]  /F = ');
	WriteLn('\\\\   /F = _');
	WriteLn('\\/    /F = ^');
	WriteLn('\\^    /F = ');
	WriteLn('\\^^   /F = ');
	WriteLn('\\v    /F = ');
	WriteLn('\\vv   /F = ');
	WriteLn('\\o    /F = ');
	WriteLn('\\oo   /F = ');
	WriteLn('\\ooo  /F = ');
	WriteLn('\\c    /F = ');
	WriteLn('\\d    /F = ');
	WriteLn('\\i    /F = ');
	WriteLn('\\m    /F = ');
	WriteLn('\\r    /F = V');
	WriteLn('\\s    /F = ');
	
	WriteLn(' /KHO = u m w y');
	WriteLn(' /KHC = v n x z');
	WriteLn(' /KH = B D');
	WriteLn(' /KH = A C');
	WriteLn(' /KH = E ^');
	WriteLn(' /KH =  J');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	WriteLn(' /KH = ');
	
	MakeKana;
	MakeNIK50;
end.
